#!/usr/bin/env python
# -*- coding:utf-8 -*-
from multiprocessing import Pool
from tqdm import tqdm
import argparse
from util import *
from kmeans import kmeans, kmeans_back
from threshold import threshold


def method_kmeans(imglist, core = 5):
    edges = [kmeans(x, core) for x in imglist]
    for pair, edge in tqdm(zip(path, edges)):
        kmeans_back(pair, edge)


def method_threshold(imglist, process = 8):
    pool = Pool(process)
    pool.map(threshold, imglist)
    pool.close()
    pool.join()


def method_newThreshold(imglist, process = 8):
    pool = Pool(process)
    pool.map(lambda x: threshold(x,'fft'), imglist)
    pool.close()
    pool.join()


def get_args():
    parser = argparse.ArgumentParser(description = 'Identify targets from background by KMeans or Threshold',
                                     formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m', '--method', metavar = 'M', type = int, default = 0,
                        help = '0 for KMeans; 1 for Threshold; 2 for newThreshold', dest = 'method')
    #parser.add_argument('-o', '--out', metavar = 'O', type = str, default = './out', help = 'Path to output',
    #                    dest = 'out')
    parser.add_argument('-c', '--core', metavar = 'C', type = int, default = 5, help = 'Num of cluster', dest = 'core')
    parser.add_argument('-p', '--process', metavar = 'P', type = int, default = 8, help = 'Num of process',
                        dest = 'process')
    return parser.parse_args()


if __name__ == '__main__':
    args = get_args()
    path = [(y, x) for y in os.listdir(args.dir) for x in filter(
        lambda x: x.endswith('.tif') and not x.endswith('dc.tif') and not x.endswith('DC.tif') and not x.endswith(
            'dc .tif'), os.listdir('img/' + y))]
    if args.method == 0:
        method_kmeans(path,args.core)
    elif args.method == 1:
        method_threshold(path,args.process)
    elif args.method == 2:
        method_newThreshold(path,args.process)
