#!/usr/bin/env python
# -*- coding:utf-8 -*-
import numpy as np
import cv2 as cv
import pandas as pd
import matplotlib.pyplot as plt
import logging
import os
from filters import fft_mask,butterworth

def remove_scratch(img):
    f = np.fft.fft2(img)
    fshift = np.fft.fftshift(f)
    fshift[100, 0:90] = 0
    fshift[100, 110:] = 0
    transed = np.abs(np.fft.ifft2(np.fft.ifftshift(fshift))).astype(np.uint8)
    return transed


def unevenLightCompensate(img, blockSize, GaussianBlur_kernelsize = 7):
    gray = img
    average = np.mean(gray)

    rows_new = int(np.ceil(gray.shape[0] / blockSize))
    cols_new = int(np.ceil(gray.shape[1] / blockSize))

    blockImage = np.zeros((rows_new, cols_new), dtype = np.float32)
    for r in range(rows_new):
        for c in range(cols_new):
            rowmin = r * blockSize
            rowmax = (r + 1) * blockSize
            if (rowmax > gray.shape[0]):
                rowmax = gray.shape[0]
            colmin = c * blockSize
            colmax = (c + 1) * blockSize
            if (colmax > gray.shape[1]):
                colmax = gray.shape[1]

            imageROI = gray[rowmin:rowmax, colmin:colmax]
            temaver = np.mean(imageROI)
            blockImage[r, c] = temaver

    blockImage = blockImage - average
    blockImage2 = cv.resize(blockImage, (gray.shape[1], gray.shape[0]), interpolation = cv.INTER_CUBIC)
    gray2 = gray.astype(np.float32)
    dst = gray2 - blockImage2
    dst = dst.astype(np.uint8)
    # dst = cv.GaussianBlur(dst, (GaussianBlur_kernelsize, GaussianBlur_kernelsize), 0)
    return dst


def pre(pair,even='clahe',filter='butter'):
    lab = pair[0]
    pic = pair[1]
    img = cv.imread('img/' + lab + '/' + pic, 0)
    even = unevenLightCompensate(img, 40) if even== 'light' else cv.createCLAHE(clipLimit = 2.0, tileGridSize = (8, 8)).apply(img)
    filtered = fft_mask(even, 10) if filter == 'fft' else butterworth(even, 10, 2)
    denoise = cv.fastNlMeansDenoising(even)
    blr = cv.GaussianBlur(denoise, (7, 7), 0)

    return {'ori': img, 'even': even, 'filtered': filtered, 'denoise': denoise, 'blr': blr}

def save_img(img_list, dir, name):
    num = len(img_list)
    plt.figure(figsize = (100, 20))
    plt.suptitle(name)

    for i, title in zip(range(num), img_list):
        img = img_list[title]
        plt.subplot(1, num, i + 1)
        plt.title(title)
        plt.imshow(img, 'gray')
    try:
        os.makedirs('out/' + dir + '/graph')
    except:
        logging.info('Existing dir: out/' + dir + '/graph')
    plt.savefig('out/' + dir + '/graph/' + name + '.png')
    plt.close()


def calcRes(img, mask, dir = 'output', name = 'output'):
    dic = get_subarea_infor(img, mask)
    df = pd.DataFrame(dic)
    try:
        os.makedirs('out/' + dir + '/csv')
    except:
        logging.info('Existing dir: out/' + dir + '/csv')
    df.to_csv('out/' + dir + '/csv/' + name + '.csv')


def get_subarea_infor(img, mask):
    area_num, labels, stats, centroids = cv.connectedComponentsWithStats(mask)

    label_group = []
    area_group = []
    mean_group = []
    std_group = []
    back_mean = []
    back_std = []

    for i in filter(lambda x: x != 0, range(area_num)):
        group = np.where(labels == i)
        img_value = img[group]
        area_tmp = len(group[0])
        mean_tmp = np.mean(img_value)
        std_tmp = np.std(img_value)

        pos = [(group[0][i], group[1][i]) for i in range(len(group[0]))]
        res = np.zeros([200, 200], np.uint8)
        for x in range(200):
            for y in range(200):
                if (x, y) in pos:
                    res[x, y] = mask[x, y]
                else:
                    res[x, y] = 0
        kernel = np.ones((17, 17), np.uint8)
        mask_background = cv.erode(255 - res, kernel)
        minimask = cv.bitwise_xor(mask_background, 255 - res)

        img_background = img[np.where(minimask != 0)]
        mean_value = np.mean(img_background)
        std_value = np.std(img_background)

        label_group.append(i)
        area_group.append(area_tmp)
        mean_group.append(mean_tmp)
        std_group.append(std_tmp)
        back_mean.append(mean_value)
        back_std.append(std_value)
    return {'Label': label_group, 'Area': area_group, 'Mean': mean_group, 'Std': std_group, 'BackMean': back_mean,
            'BackStd': back_std}
